@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/apm/apm_driver.c,v 1.65 2003/10/30 17:36:57 tsi Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -7,21 +6,28 @@
 #include "apm.h"
 #include "xf86cmap.h"
 #include "shadowfb.h"
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include "xf86Resources.h"
-#include "xf86int10.h"
 #include "xf86RAC.h"
+#endif
+#include "xf86int10.h"
 #include "vbe.h"
 
 #include "opaque.h"
+#ifdef HAVE_XEXTPROTO_71
+#include <X11/extensions/dpmsconst.h>
+#else
 #define DPMS_SERVER
 #include <X11/extensions/dpms.h>
+#endif
 
-#define VERSION			4000
+
+#define APM_VERSION		4000
 #define APM_NAME		"APM"
 #define APM_DRIVER_NAME		"apm"
-#define APM_MAJOR_VERSION       1
-#define APM_MINOR_VERSION       0
-#define APM_PATCHLEVEL          0
+#define APM_MAJOR_VERSION       PACKAGE_VERSION_MAJOR
+#define APM_MINOR_VERSION       PACKAGE_VERSION_MINOR
+#define APM_PATCHLEVEL          PACKAGE_VERSION_PATCHLEVEL
 #ifndef PCI_CHIP_AT3D
 #define PCI_CHIP_AT3D	0x643D
 #endif
@@ -54,12 +60,13 @@ static void	ApmDisplayPowerManagementSet(ScrnInfoPtr pScrn,
 					     int flags);
 static void	ApmProbeDDC(ScrnInfoPtr pScrn, int index);
 
-
+#ifdef XF86RUSH
 int ApmPixmapIndex = -1;
 static unsigned long ApmGeneration = 0;
+#endif
 
 _X_EXPORT DriverRec APM = {
-	VERSION,
+	APM_VERSION,
 	APM_DRIVER_NAME,
 	ApmIdentify,
 	ApmProbe,
@@ -82,11 +89,6 @@ static PciChipsets ApmPciChipsets[] = {
     { -1,			-1,		RES_UNDEFINED }
 };
 
-static IsaChipsets ApmIsaChipsets[] = {
-    { PCI_CHIP_AP6422,	RES_EXCLUSIVE_VGA},
-    {-1,		RES_UNDEFINED}
-};
-
 typedef enum {
     OPTION_SET_MCLK,
     OPTION_SW_CURSOR,
@@ -132,98 +134,6 @@ static const OptionInfoRec ApmOptions[] =
 	{0}, FALSE}
 };
 
-/*
- * List of symbols from other modules that this module references.  This
- * list is used to tell the loader that it is OK for symbols here to be
- * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
- * unresolved symbols that are not required.
- */
-
-static const char *vgahwSymbols[] = {
-    "vgaHWBlankScreen",
-    "vgaHWCursor",
-    "vgaHWFreeHWRec",
-    "vgaHWGetHWRec",
-    "vgaHWGetIOBase",
-    "vgaHWInit",
-    "vgaHWLock",
-    "vgaHWMapMem",
-    "vgaHWProtect",
-    "vgaHWRestore",
-    "vgaHWSave",
-    "vgaHWSetMmioFuncs",
-    "vgaHWUnlock",
-    NULL
-};
-
-static const char *xaaSymbols[] = {
-    "XAACreateInfoRec",
-    "XAACursorInfoRec",
-    "XAACursorInit",
-    "XAADestroyInfoRec",
-    "XAAGlyphScanlineFuncLSBFirst",
-    "XAAInit",
-    "XAAReverseBitOrder",
-    "XAAStippleScanlineFuncMSBFirst",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-#ifdef XFree86LOADER
-static const char *vbeSymbols[] = {
-    "VBEInit",
-    "vbeDoEDID",
-    "vbeFree",
-    NULL
-};
-#endif
-
-static const char *ddcSymbols[] = {
-    "xf86DoEDID_DDC1",
-    "xf86DoEDID_DDC2",
-    "xf86PrintEDID",
-    NULL
-};
-
-static const char *i2cSymbols[] = {
-    "xf86CreateI2CBusRec",
-    "xf86I2CBusInit",
-    NULL
-};
-
-static const char *shadowSymbols[] = {
-    "ShadowFBInit",
-    NULL
-};
-
-#ifdef XFree86LOADER
-static const char *miscfbSymbols[] = {
-    "xf1bppScreenInit",
-    "xf4bppScreenInit",
-    NULL
-};
-#endif
-
-static const char *fbSymbols[] = {
-    "fbPictureInit",
-    "fbScreenInit",
-    NULL
-};
-
-static const char *int10Symbols[] = {
-    "xf86Free10",
-    "xf86InitInt10",
-    NULL
-};
-
 #ifdef XFree86LOADER
 
 static XF86ModuleVersionInfo apmVersRec = {
@@ -256,11 +166,6 @@ apmSetup(pointer module, pointer opts, int *errmaj, int *errmain)
 	setupDone = TRUE;
 	xf86AddDriver(&APM, module, 0);
 
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols, 
-			  miscfbSymbols, ramdacSymbols, vbeSymbols,
-			  ddcSymbols, i2cSymbols, shadowSymbols, 
-			  int10Symbols, NULL);
-
 	return (pointer)1;
     }
     else {
@@ -324,59 +229,10 @@ ApmAvailableOptions(int chipid, int busid)
     return ApmOptions;
 }
 
-static int
-ApmFindIsaDevice(GDevPtr dev)
-{
-    char	save = rdinx(0x3C4, 0x10);
-    int		i;
-    int		apmChip = -1;
-
-    /*
-     * Start by probing the VGA chipset.
-     */
-    outw(0x3C4, 0x1210);
-    if (rdinx(0x3C4, 0x11) == 'P' && rdinx(0x3C4, 0x12) == 'r' &&
-	rdinx(0x3C4, 0x13) == 'o') {
-	char	id_ap6420[] = "6420";
-	char	id_ap6422[] = "6422";
-	char	id_at24[]   = "6424";
-	char	id_at3d[]   = "AT3D";
-	char	idstring[]  = "    ";
-
-	/*
-	 * Must be an Alliance !!!
-	 */
-	for (i = 0; i < 4; i++)
-	    idstring[i] = rdinx(0x3C4, 0x14 + i);
-	if (!memcmp(id_ap6420, idstring, 4) ||
-	    !memcmp(id_ap6422, idstring, 4))
-	    apmChip = AP6422;
-	else if (!memcmp(id_at24, idstring, 4))
-	    apmChip = AT24;
-	else if (!memcmp(id_at3d, idstring, 4))
-	    apmChip = AT3D;
-	if (apmChip >= 0) {
-	    int	apm_xbase;
-
-	    apm_xbase = (rdinx(0x3C4, 0x1F) << 8) | rdinx(0x3C4, 0x1E);
-
-	    if (!(wrinx(0x3C4, 0x1D, 0xCA >> 2), inb(apm_xbase + 2))) {
-		/*
-		 * TODO Not PCI
-		 */
-	    }
-
-	}
-    }
-    wrinx(0x3C4, 0x10, save);
-
-    return apmChip;
-}
-
 static void
 ApmAssignFPtr(ScrnInfoPtr pScrn)
 {
-    pScrn->driverVersion	= VERSION;
+    pScrn->driverVersion	= APM_VERSION;
     pScrn->driverName		= APM_DRIVER_NAME;
     pScrn->name			= APM_NAME;
     pScrn->Probe		= ApmProbe;
@@ -411,9 +267,11 @@ ApmProbe(DriverPtr drv, int flags)
      * file info to override any contradictions.
      */
 
+#ifndef XSERVER_LIBPCIACCESS
     if (xf86GetPciVideoInfo() == NULL) {
 	return FALSE;
     }
+#endif
     numUsed = xf86MatchPciInstances(APM_NAME, PCI_VENDOR_ALLIANCE,
 		    ApmChipsets, ApmPciChipsets, DevSections, numDevSections,
 		    drv, &usedChips);
@@ -441,26 +299,7 @@ ApmProbe(DriverPtr drv, int flags)
 	}
     }
 
-    /* Check for non-PCI cards */
-    numUsed = xf86MatchIsaInstances(APM_NAME, ApmChipsets,
-			ApmIsaChipsets, drv, ApmFindIsaDevice, DevSections,
-			numDevSections, &usedChips);
-    if (numUsed > 0) {
-	if (flags & PROBE_DETECT)
-	    foundScreen = TRUE;
-	else for (i = 0; i < numUsed; i++) {
-	    ScrnInfoPtr pScrn = NULL;
-	    if ((pScrn = xf86ConfigIsaEntity(pScrn, 0, usedChips[i],
-					     ApmIsaChipsets, NULL, NULL, NULL,
-					     NULL, NULL))) {
-	    /*
-	     * Fill in what we can of the ScrnInfoRec
-	     */
-	    ApmAssignFPtr(pScrn);
-	    foundScreen = TRUE;
-	    }
-	}
-    }
+
     xfree(DevSections);
     return foundScreen;
 }
@@ -550,12 +389,16 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     pEnt = pApm->pEnt	= xf86GetEntityInfo(pScrn->entityList[0]);
     if (pEnt->location.type == BUS_PCI) {
 	pApm->PciInfo	= xf86GetPciInfoForEntity(pEnt->index);
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->PciTag	= pciTag(pApm->PciInfo->bus, pApm->PciInfo->device,
 				 pApm->PciInfo->func);
+#endif
     }
     else {
 	pApm->PciInfo	= NULL;
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->PciTag	= 0;
+#endif
     }
 
     if (flags & PROBE_DETECT) {
@@ -567,8 +410,6 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     if (!xf86LoadSubModule(pScrn, "vgahw"))
 	return FALSE;
 
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
-
     /*
      * Allocate a vgaHWRec
      */
@@ -653,10 +494,15 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	/* Default to 8 */
 	pScrn->rgbBits = 8;
     }
+#ifndef XSERVER_LIBPCIACCESS
+    /* you're getting a linear framebuffer with pciaccess */
     if (xf86ReturnOptValBool(pApm->Options, OPTION_NOLINEAR, FALSE)) {
 	pApm->noLinear = TRUE;
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "No linear framebuffer\n");
     }
+#else
+    pApm->noLinear = FALSE;
+#endif
     from = X_DEFAULT;
     pApm->hwCursor = FALSE;
     if (xf86GetOptValBool(pApm->Options, OPTION_HW_CURSOR, &pApm->hwCursor))
@@ -778,7 +624,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     } else {
 	from = X_PROBED;
 	if (pApm->PciInfo)
-	    pApm->Chipset = pApm->PciInfo->chipType;
+	    pApm->Chipset = PCI_DEV_DEVICE_ID(pApm->PciInfo);
 	else
 	    pApm->Chipset = pEnt->chipset;
 	pScrn->chipset = (char *)xf86TokenToString(ApmChipsets, pApm->Chipset);
@@ -794,7 +640,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipRev override: %d\n",
 		   pApm->ChipRev);
     } else if (pApm->PciInfo) {
-	pApm->ChipRev = pApm->PciInfo->chipRev;
+        pApm->ChipRev = PCI_DEV_REVISION(pApm->PciInfo);
     }
 
     /*
@@ -818,7 +664,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	pApm->LinAddress = pEnt->device->MemBase;
 	from = X_CONFIG;
     } else if (pApm->PciInfo) {
-	pApm->LinAddress = pApm->PciInfo->memBase[0] & 0xFF800000;
+        pApm->LinAddress = PCI_REGION_BASE(pApm->PciInfo, 0, REGION_MEM) & 0xFF800000;
 	from = X_PROBED;
     } else {
 	/*
@@ -832,9 +678,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	       (unsigned long)pApm->LinAddress);
 
     if (xf86LoadSubModule(pScrn, "ddc")) {
-	xf86LoaderReqSymLists(ddcSymbols, NULL);
 	if (xf86LoadSubModule(pScrn, "i2c")) {
-	    xf86LoaderReqSymLists(i2cSymbols, NULL);
 	    pApm->I2C = TRUE;
 	}
     }
@@ -860,13 +704,13 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     if (xf86LoadSubModule(pScrn, "int10")) {
 	void	*ptr;
 
-	xf86LoaderReqSymLists(int10Symbols, NULL);
 	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"initializing int10\n");
 	ptr = xf86InitInt10(pEnt->index);
 	if (ptr)
 	    xf86FreeInt10(ptr);
     }
 
+#ifndef XSERVER_LIBPCIACCESS
     xf86RegisterResources(pEnt->index, NULL, ResNone);
     xf86SetOperatingState(resVga, pEnt->index, ResDisableOpr);
     pScrn->racMemFlags = 0;	/* For noLinear, access to 0xA0000 */
@@ -874,7 +718,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	pScrn->racIoFlags = 0;
     else
 	pScrn->racIoFlags = RAC_COLORMAP | RAC_VIEWPORT;
-
+#endif
     if (pEnt->device->videoRam != 0) {
 	pScrn->videoRam = pEnt->device->videoRam;
 	from = X_CONFIG;
@@ -883,9 +727,24 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	/*unsigned long		save;*/
 	volatile unsigned char	*LinMap;
 
+#ifndef XSERVER_LIBPCIACCESS
 	LinMap = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
 				     pApm->PciTag, pApm->LinAddress,
 				     pApm->LinMapSize);
+#else
+	{
+	    void** result = (void**)&LinMap;
+	    int err = pci_device_map_range(pApm->PciInfo,
+					   pApm->LinAddress,
+					   pApm->LinMapSize,
+					   PCI_DEV_MAP_FLAG_WRITABLE,
+					   result);
+	    
+	    if (err)
+		return FALSE;
+	}
+#endif
+
 	/*save = pciReadLong(pApm->PciTag, PCI_CMD_STAT_REG);
 	pciWriteLong(pApm->PciTag, PCI_CMD_STAT_REG, save | PCI_CMD_MEM_ENABLE);*/
 	d9 = LinMap[0xFFECD9];
@@ -945,8 +804,10 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	}
 	if (0 && !MonInfo)
 	    MonInfo = xf86DoEDID_DDC1(pScrn->scrnIndex,vgaHWddc1SetSpeed,ddc1Read);
-	if (MonInfo)
+	if (MonInfo) {
 	    xf86PrintEDID(MonInfo);
+	    xf86SetDDCproperties(pScrn, MonInfo);
+	}
 	pScrn->monitor->DDC = MonInfo;
     }
 
@@ -1131,14 +992,6 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Load bpp-specific modules */
     switch (pScrn->bitsPerPixel) {
-    case 1:
-	mod = "xf1bpp";
-	req = "xf1bppScreenInit";
-	break;
-    case 4:
-	mod = "xf4bpp";
-	req = "xf4bppScreenInit";
-	break;
     case 8:
     case 16:
     case 24:
@@ -1152,21 +1005,12 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
 
-    if (mod) {
-	if (req) {
-	    xf86LoaderReqSymbols(req, NULL);
-	} else {
-	    xf86LoaderReqSymLists(fbSymbols, NULL);
-	}
-    }
-
     /* Load XAA if needed */
     if (!pApm->NoAccel) {
 	if (!xf86LoadSubModule(pScrn, "xaa")) {
 	    ApmFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
     }
 
     /* Load ramdac if needed */
@@ -1175,7 +1019,6 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	    ApmFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
     /* Load shadowfb if needed */
@@ -1184,7 +1027,6 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	    ApmFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
     }
 
     pApm->CurrentLayout.displayWidth	= pScrn->virtualX;
@@ -1211,10 +1053,27 @@ ApmMapMem(ScrnInfoPtr pScrn)
     APMDECL(pScrn);
     vgaHWPtr	hwp = VGAHWPTR(pScrn);
 
+#ifndef XSERVER_LIBPCIACCESS
     pApm->LinMap = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
 				 pApm->PciTag,
 				 (unsigned long)pApm->LinAddress,
 				 pApm->LinMapSize);
+#else
+    {
+	void** result = (void**)&pApm->LinMap;
+	int err = pci_device_map_range(pApm->PciInfo,
+				       pApm->LinAddress,
+				       pApm->LinMapSize,
+				       PCI_DEV_MAP_FLAG_WRITABLE |
+				       PCI_DEV_MAP_FLAG_WRITE_COMBINE,
+				       result);
+	
+	if (err) 
+	    return FALSE;
+    }
+#endif
+
+
     if (pApm->LinMap == NULL)
 	return FALSE;
 
@@ -1877,10 +1736,12 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     /* Map the chip memory and MMIO areas */
     if (pApm->noLinear) {
-	pApm->saveCmd = pciReadLong(pApm->PciTag, PCI_CMD_STAT_REG);
-	pciWriteLong(pApm->PciTag, PCI_CMD_STAT_REG, pApm->saveCmd | (PCI_CMD_IO_ENABLE|PCI_CMD_MEM_ENABLE));
+	PCI_READ_LONG(pApm->PciInfo, &pApm->saveCmd, PCI_CMD_STAT_REG);
+	PCI_WRITE_LONG(pApm->PciInfo, pApm->saveCmd | (PCI_CMD_IO_ENABLE | PCI_CMD_MEM_ENABLE), PCI_CMD_STAT_REG);
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->FbBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
 				 pApm->PciTag, 0xA0000, 0x10000);
+#endif
     }
     else
 	if (!ApmMapMem(pScrn))
@@ -1946,18 +1807,6 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     miSetPixmapDepths();
 
     switch (pScrn->bitsPerPixel) {
-    case 1:
-	ret = xf1bppScreenInit(pScreen, FbBase,
-			pScrn->virtualX, pScrn->virtualY,
-			pScrn->xDpi, pScrn->yDpi,
-			pScrn->displayWidth);
-	break;
-    case 4:
-	ret = xf4bppScreenInit(pScreen, FbBase,
-			pScrn->virtualX, pScrn->virtualY,
-			pScrn->xDpi, pScrn->yDpi,
-			pScrn->displayWidth);
-	break;
     case 8:
     case 16:
     case 24:
@@ -2064,6 +1913,7 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);
     }
 
+#ifdef XF86RUSH
     if (ApmGeneration != serverGeneration) {
 	if ((ApmPixmapIndex = AllocatePixmapPrivateIndex()) < 0)
 	    return FALSE;
@@ -2072,6 +1922,7 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     if (!AllocatePixmapPrivate(pScreen, ApmPixmapIndex, sizeof(ApmPixmapRec)))
 	return FALSE;
+#endif
 
     /* Done */
     return TRUE;
